@@ -3,7 +3,7 @@ Tests of the code in uncertainties.umath.
 
 These tests can be run through the Nose testing framework.
 
-(c) 2010 by Eric O. LEBIGOT (EOL).
+(c) 2010-2013 by Eric O. LEBIGOT (EOL).
 """
 
 from __future__ import division
@@ -29,16 +29,16 @@ def test_fixed_derivatives_math_funcs():
     This comparison is useful for derivatives that are analytical.
     """
 
-    for name in umath.many_scalar_to_scalar_funcs:
+    for name in umath.many_scalars_to_scalar_funcs:
         # print "Checking %s..." % name
         func = getattr(umath, name)
         # Numerical derivatives of func: the nominal value of func() results
         # is used as the underlying function:
         numerical_derivatives = uncertainties.NumericalDerivatives(
-            lambda *args: func(*args).nominal_value)
+            lambda *args: func(*args))
         test_uncertainties._compare_derivatives(func, numerical_derivatives)
 
-    # Functions that are not in umath.many_scalar_to_scalar_funcs:
+    # Functions that are not in umath.many_scalars_to_scalar_funcs:
 
     ##
     # modf(): returns a tuple:
@@ -50,11 +50,11 @@ def test_fixed_derivatives_math_funcs():
     test_uncertainties._compare_derivatives(
         frac_part_modf,
         uncertainties.NumericalDerivatives(
-            lambda x: frac_part_modf(x).nominal_value))
+            lambda x: frac_part_modf(x)))
     test_uncertainties._compare_derivatives(
         int_part_modf,
         uncertainties.NumericalDerivatives(
-            lambda x: int_part_modf(x).nominal_value))
+            lambda x: int_part_modf(x)))
     
     ##
     # frexp(): returns a tuple:
@@ -66,11 +66,11 @@ def test_fixed_derivatives_math_funcs():
     test_uncertainties._compare_derivatives(
         mantissa_frexp,
         uncertainties.NumericalDerivatives(
-            lambda x: mantissa_frexp(x).nominal_value))
+            lambda x: mantissa_frexp(x)))
     test_uncertainties._compare_derivatives(
         exponent_frexp,
         uncertainties.NumericalDerivatives(
-            lambda x: exponent_frexp(x).nominal_value))
+            lambda x: exponent_frexp(x)))
 
 def test_compound_expression():
     """
@@ -111,11 +111,13 @@ def test_monte_carlo_comparison():
         import numpy
         import numpy.random
     except ImportError:
-        raise Exception("Test not performed because NumPy is not available")
+        import warnings
+        warnings.warn("Test not performed because NumPy is not available")
+        return
 
     # Works on numpy.arrays of Variable objects (whereas umath.sin()
     # does not):
-    sin_uarrayncert = numpy.vectorize(umath.sin, otypes=[object])
+    sin_uarray_uncert = numpy.vectorize(umath.sin, otypes=[object])
     
     # Example expression (with correlations, and multiple variables combined
     # in a non-linear way):
@@ -125,7 +127,7 @@ def test_monte_carlo_comparison():
         """
         # The uncertainty due to x is about equal to the uncertainty
         # due to y:
-        return 10 * x**2 - x * sin_uarrayncert(y**3)
+        return 10 * x**2 - x * sin_uarray_uncert(y**3)
 
     x = uncertainties.ufloat((0.2, 0.01))
     y = uncertainties.ufloat((10, 0.001))
@@ -204,7 +206,7 @@ def test_math_module():
 
     # Python >=2.6 functions:
 
-    if sys.version_info[:2] >= (2, 6):
+    if sys.version_info >= (2, 6):
     
         # factorial() must not be "damaged" by the umath module, so as 
         # to help make it a drop-in replacement for math (even though 
@@ -223,3 +225,71 @@ def test_math_module():
         # fsum is special because it does not take a fixed number of
         # variables:
         assert umath.fsum([x, x]).nominal_value == -3
+
+    # The same exceptions should be generated when numbers with uncertainties
+    # are used:
+
+    ## !! The Nose testing framework seems to catch an exception when
+    ## it is aliased: "exc = OverflowError; ... except exc:..."
+    ## surprisingly catches OverflowError. So, tests are written in a
+    ## version-specific manner (until the Nose issue is resolved).
+
+    if sys.version_info < (2, 6):
+            
+        try:
+            math.log(0)
+        except OverflowError, err_math:  # "as", for Python 2.6+
+            pass
+        else:
+            raise Exception('OverflowError exception expected')
+        try:
+            umath.log(0)
+        except OverflowError, err_ufloat:  # "as", for Python 2.6+
+            assert err_math.args == err_ufloat.args
+        else:
+            raise Exception('OverflowError exception expected')
+        try:
+            umath.log(uncertainties.ufloat((0, 0)))
+        except OverflowError, err_ufloat:  # "as", for Python 2.6+
+            assert err_math.args == err_ufloat.args
+        else:
+            raise Exception('OverflowError exception expected')
+        try:
+            umath.log(uncertainties.ufloat((0, 1)))
+        except OverflowError, err_ufloat:  # "as", for Python 2.6+
+            assert err_math.args == err_ufloat.args
+        else:
+            raise Exception('OverflowError exception expected')
+
+    elif sys.version_info < (3,):
+
+        try:
+            math.log(0)
+        except ValueError, err_math:
+            pass
+        else:
+            raise Exception('ValueError exception expected')
+        try:
+            umath.log(0)
+        except ValueError, err_ufloat:
+            assert err_math.args == err_ufloat.args
+        else:
+            raise Exception('ValueError exception expected')
+        try:
+            umath.log(uncertainties.ufloat((0, 0)))
+        except ValueError, err_ufloat:
+            assert err_math.args == err_ufloat.args
+        else:
+            raise Exception('ValueError exception expected')
+        try:
+            umath.log(uncertainties.ufloat((0, 1)))
+        except ValueError, err_ufloat:
+            assert err_math.args == err_ufloat.args
+        else:
+            raise Exception('ValueError exception expected')
+        
+    else:  # Python 3+
+        
+        # !!! The tests should be made to work with Python 3 too!
+        pass
+    
