@@ -1,9 +1,11 @@
+# coding=utf-8
+
 """
 Tests of the code in uncertainties/__init__.py.
 
 These tests can be run through the Nose testing framework.
 
-(c) 2010 by Eric O. LEBIGOT (EOL).
+(c) 2010-2013 by Eric O. LEBIGOT (EOL).
 """
 
 from __future__ import division
@@ -12,9 +14,11 @@ from __future__ import division
 import copy
 import weakref
 import math
+import random
+import sys
 
 # 3rd-party modules
-import nose.tools
+# import nose.tools
 
 # Local modules
 
@@ -23,13 +27,18 @@ from uncertainties import ufloat, AffineScalarFunc
 
 from uncertainties import __author__
 
+# The following information is useful for making sure that the right
+# version of Python is running the tests (for instance with the Travis
+# Continuous Integration system):
+print "Testing with Python", sys.version
+
 ###############################################################################
 
 # Utilities for unit testing
 
 def _numbers_close(x, y, tolerance=1e-6):
     """
-    Returns True if the numbers are close enough.
+    Returns True if the given (real) numbers are close enough.
 
     The given tolerance is the relative difference allowed, or the absolute
     difference, if one of the numbers is 0.
@@ -49,6 +58,7 @@ def _numbers_close(x, y, tolerance=1e-6):
 class DerivativesDiffer(Exception):
     pass
 
+    
 def _compare_derivatives(func, numerical_derivatives,
                          num_args_list=None):
     """
@@ -64,8 +74,6 @@ def _compare_derivatives(func, numerical_derivatives,
     Tests are done on random arguments.
     """
 
-    import random
-
     # print "Testing", func.__name__
 
     if not num_args_list: 
@@ -118,9 +126,13 @@ def _compare_derivatives(func, numerical_derivatives,
                 args = [
                     random.choice(range(-10, 10))
                     if arg_num in integer_arg_nums
-                    else uncertainties.Variable(random.random()*10-5, 0)
+                    else uncertainties.Variable(random.random()*4-2, 0)
                     for arg_num in range(num_args)]
 
+                # 'args', but as scalar values:
+                args_scalar = [uncertainties.nominal_value(v)
+                               for v in args]
+
                 func_approx = func(*args)
 
                 # Some functions yield simple Python constants, after
@@ -129,22 +141,23 @@ def _compare_derivatives(func, numerical_derivatives,
                 if isinstance(func_approx, AffineScalarFunc):
                     
                     # We compare all derivatives:
-                    for (arg_num, (arg, numerical_deriv)) in \
-                        enumerate(zip(args, numerical_derivatives)):
+                    for (arg_num, (arg, numerical_deriv)) in (
+                        enumerate(zip(args, numerical_derivatives))):
 
                         # Some arguments might not be differentiable:
                         if isinstance(arg, int):
                             continue
 
                         fixed_deriv_value = func_approx.derivatives[arg]
-                        num_deriv_value = numerical_deriv(*args)
+                        
+                        num_deriv_value = numerical_deriv(*args_scalar)
 
                         # This message is useful: the user can see that
                         # tests are really performed (instead of not being
                         # performed, silently):
                         print "Testing %s at %s, arg #%d" % (
                             func.__name__, args, arg_num)
-
+                        
                         if not _numbers_close(fixed_deriv_value,
                                               num_deriv_value, 1e-4):
 
@@ -156,8 +169,8 @@ def _compare_derivatives(func, numerical_derivatives,
                                 raise DerivativesDiffer(
                                     "Derivative #%d of function '%s' may be"
                                     " wrong: at args = %s,"
-                                    " value obtained = %f,"
-                                    " while numerical approximation = %f."
+                                    " value obtained = %.16f,"
+                                    " while numerical approximation = %.16f."
                                     % (arg_num, func.__name__, args,
                                        fixed_deriv_value, num_deriv_value))
 
@@ -180,7 +193,7 @@ def _compare_derivatives(func, numerical_derivatives,
                 break
 
 ###############################################################################
-
+            
 # Test of correctness of the fixed (usually analytical) derivatives:
 def test_fixed_derivatives_basic_funcs():
     """
@@ -196,10 +209,13 @@ def test_fixed_derivatives_basic_funcs():
         """
 
         op_string = "__%s__" % op
-        # print "Checking %s..." % op_string
         func = getattr(AffineScalarFunc, op_string)
         numerical_derivatives = uncertainties.NumericalDerivatives(
-            lambda *args: func(*args).nominal_value)
+            # The __neg__ etc. methods of AffineScalarFunc only apply,
+            # by definition, to AffineScalarFunc objects: we first map
+            # possible scalar arguments (used for calculating
+            # derivatives) to AffineScalarFunc objects:
+            lambda *args: func(*map(uncertainties.to_affine_scalar, args)))
         _compare_derivatives(func, numerical_derivatives, [num_args])
 
     # Operators that take 1 value:
@@ -207,7 +223,7 @@ def test_fixed_derivatives_basic_funcs():
         check_op(op, 1)
 
     # Operators that take 2 values:
-    for op in uncertainties._ops_with_reflection.keys():
+    for op in uncertainties._modified_ops_with_reflection:
         check_op(op, 2)
 
 # Additional, more complex checks, for use with the nose unit testing
@@ -260,7 +276,22 @@ def test_copy():
     gc.collect()
 
     assert y in y.derivatives.keys()
+
+## Classes for the pickling tests (put at the module level, so that
+## they can be unpickled):
     
+# Subclass without slots:
+class NewVariable_dict(uncertainties.Variable):
+    pass
+
+# Subclass with slots defined by a tuple:
+class NewVariable_slots_tuple(uncertainties.Variable):
+    __slots__ = ('new_attr',)
+
+# Subclass with slots defined by a string:
+class NewVariable_slots_str(uncertainties.Variable):
+    __slots__ = 'new_attr'
+        
 def test_pickling():
     "Standard pickle module integration."
 
@@ -279,7 +310,46 @@ def test_pickling():
     # Correlations must be preserved:
     assert f_unpickled - x_unpickled2 - x_unpickled2 == 0
     
+    ## Tests with subclasses:
+
+    for subclass in (NewVariable_dict, NewVariable_slots_tuple,
+                     NewVariable_slots_str):
+        
+        x = subclass(3, 0.14)
+
+        # Pickling test with possibly uninitialized slots:
+        pickle.loads(pickle.dumps(x))
+        
+        # Unpickling test:
+        x.new_attr = 'New attr value'
+        x_unpickled = pickle.loads(pickle.dumps(x))
+        # Must exist (From the slots of the parent class):        
+        x_unpickled.nominal_value
+        x_unpickled.new_attr  # Must exist    
+
+    ##
+        
+    # Corner case test: when an attribute is present both in __slots__
+    # and in __dict__, it is first looked up from the slots
+    # (references:
+    # http://docs.python.org/2/reference/datamodel.html#invoking-descriptors,
+    # http://stackoverflow.com/a/15139208/42973). As a consequence,
+    # the pickling process must pickle the correct value (i.e., not
+    # the value from __dict__):
+    x = NewVariable_dict(3, 0.14)
+    x._nominal_value = 'in slots'
+    # Corner case: __dict__ key which is also a slot name (it is
+    # shadowed by the corresponding slot, so this is very unusual,
+    # though):
+    x.__dict__['_nominal_value'] = 'in dict'
+    # Additional __dict__ attribute:
+    x.dict_attr = 'dict attribute'
     
+    x_unpickled = pickle.loads(pickle.dumps(x))
+    # We make sure that the data is still there and untouched:
+    assert x_unpickled._nominal_value == 'in slots'
+    assert x_unpickled.__dict__ == x.__dict__
+        
 def test_int_div():
     "Integer division"
     # We perform all operations on floats, because derivatives can
@@ -474,12 +544,12 @@ def test_basic_access_to_data():
     assert y.error_components()[x] == 5  # New error contribution!
 
     # Calculation of deviations in units of the standard deviations:
-    assert 10/x.std_dev() == x.position_in_sigmas(10 + x.nominal_value)
+    assert 10/x.std_dev() == x.std_score(10 + x.nominal_value)
 
     # "In units of the standard deviation" is not always meaningfull:
     x.set_std_dev(0)
     try:
-        x.position_in_sigmas(1)
+        x.std_score(1)
     except ValueError:
         pass  # Normal behavior
 
@@ -591,19 +661,6 @@ def test_access_to_std_dev():
     assert uncertainties.std_dev([]) == 0
     assert uncertainties.std_dev(None) == 0
     
-def test_legacy():
-    "Test of legacy code"
-    x = (1, 0.1)
-
-    old1 = uncertainties.NumberWithUncert(x)  # Warnings are normal
-    old2 = uncertainties.num_with_uncert(x)  # Warnings are normal
-    new = uncertainties.ufloat(x)
-    
-    assert old1.nominal_value == new.nominal_value
-    assert old2.nominal_value == new.nominal_value
-    assert old1.std_dev() == new.std_dev()
-    assert old2.std_dev() == new.std_dev()
-
 ###############################################################################
 
 def test_covariances():
@@ -620,6 +677,101 @@ def test_covariances():
     # Non-diagonal elements:
     assert _numbers_close(covs[0][1], -0.02)
 
+###############################################################################
+    
+def test_power():
+    '''
+    Checks special cases of x**p.
+
+    The value x = 0 is special, as are positive, null and negative
+    and integral values of p.
+    '''
+
+    zero = ufloat((0, 0))
+    one = ufloat((1, 0))
+    p = ufloat((0.3, 0.01))
+
+    # assert 0**p == 0  # !!! Should pass
+    # assert zero**p == 0  # !!! Should pass
+
+    # Should raise the same errors as float operations:
+    try:
+        0**(-p)
+    except ZeroDivisionError:
+        pass
+    else:
+        raise Exception('An proper exception should have been raised')
+
+    try:
+        zero**(-p)
+    except ZeroDivisionError:
+        pass
+    else:
+        raise Exception('An proper exception should have been raised')
+
+    if sys.version_info >= (2, 6):
+        
+        # Reference: http://docs.python.org/library/math.html#math.pow
+        
+        # …**0 == 1.0:
+        assert p**0 == 1.0        
+        # assert zero**0 == 1.0  # !!! Should pass
+        assert (-p)**0 == 1.0
+        # …**zero:
+        # assert (-10.3)**zero == 1.0  # !!! Should pass
+        # assert 0**zero == 1.0  # !!! Should pass
+        assert 0.3**zero == 1.0
+        # assert float('nan')**zero == 1.0  # !!! Should pass
+        # assert (-p)**zero == 1.0  # !!! Should pass
+        # assert zero**zero == 1.0  # !!! Should pass
+        assert p**zero == 1.0
+
+        # one**… == 1.0
+        assert one**-3 == 1.0
+        assert one**-3.1 == 1.0
+        assert one**0 == 1.0
+        assert one**3 == 1.0
+        assert one**3.1 == 1.0
+        # assert one**float('nan') == 1.0  # !!! Should pass
+        # … with two numbers with uncertainties:
+        assert one**(-p) == 1.0
+        assert one**zero == 1.0
+        assert one**p == 1.0
+        # 1**… == 1.0:
+        assert 1.**(-p) == 1.0
+        assert 1.**zero == 1.0
+        assert 1.**p == 1.0
+        
+        
+    # Negative numbers with unceratinty can be exponentiated to an integral
+    # power:
+    assert (ufloat((-1.1, 0.1))**-9).nominal_value == (-1.1)**-9
+
+    # Case of numbers with no uncertainty: should give the same result
+    # as numbers with uncertainties:
+    assert ufloat((-1, 0))**9 == (-1)**9
+    assert ufloat((-1.1, 0))**9 == (-1.1)**9
+    
+    # Negative numbers cannot be raised to a non-integral power, in
+    # Python 2 (in Python 3, complex numbers are returned; this cannot
+    # (yet) be represented in the uncertainties package, because it
+    # does not handle complex numbers):
+    if sys.version_info < (3,):
+        try:
+            ufloat((-1, 0))**9.1
+        except Exception, err_ufloat:  # "as", for Python 2.6+
+            pass
+        else:
+            raise Exception('An exception should have been raised')
+        try:
+            (-1)**9.1
+        except Exception, err_float:  # "as" for Python 2.6+
+            # UFloat and floats should raise the same error:
+            assert err_ufloat.args == err_float.args
+        else:
+            raise Exception('An exception should have been raised')
+
+    
 ###############################################################################
 
 # The tests below require NumPy, which is an optional package:
@@ -700,7 +852,10 @@ else:
         assert numpy.all((x >= numpy.arange(3)) == [True, False, False])
 
     def test_correlated_values():
-        "Correlated variables."
+        """
+        Correlated variables.
+        Test through the input of the (full) covariance matrix.
+        """
 
         u = uncertainties.ufloat((1, 0.1))
         cov = uncertainties.covariance_matrix([u])
@@ -721,6 +876,11 @@ else:
 
         covs = uncertainties.covariance_matrix([x, y, z])
 
+        # Test of the diagonal covariance elements:
+        assert matrices_close(
+            numpy.array([v.std_dev()**2 for v in (x, y, z)]),
+            numpy.array(covs).diagonal())
+        
         # "Inversion" of the covariance matrix: creation of new
         # variables:
         (x_new, y_new, z_new) = uncertainties.correlated_values(
@@ -751,14 +911,65 @@ else:
         # Covariance matrices:
         cov_matrix = uncertainties.covariance_matrix([u, v, sum_value])
 
-        # Correlated variables can be constructed from a covariance matrix, if
-        # NumPy is available:
+        # Correlated variables can be constructed from a covariance
+        # matrix, if NumPy is available:
         (u2, v2, sum2) = uncertainties.correlated_values(
             [x.nominal_value for x in [u, v, sum_value]],
             cov_matrix)
 
         # matrices_close() is used instead of _numbers_close() because
         # it compares uncertainties too:
+        assert matrices_close(numpy.array([u]), numpy.array([u2]))
+        assert matrices_close(numpy.array([v]), numpy.array([v2]))
+        assert matrices_close(numpy.array([sum_value]), numpy.array([sum2]))
         assert matrices_close(numpy.array([0]),
                               numpy.array([sum2-(u2+2*v2)]))
 
+
+    def test_correlated_values_correlation_mat():
+        '''
+        Tests the input of correlated value.
+
+        Test through their correlation matrix (instead of the
+        covariance matrix).
+        '''
+        
+        x = ufloat((1, 0.1))
+        y = ufloat((2, 0.3))
+        z = -3*x+y
+
+        cov_mat = uncertainties.covariance_matrix([x, y, z])
+
+        std_devs = numpy.sqrt(numpy.array(cov_mat).diagonal())
+        
+        corr_mat = cov_mat/std_devs/std_devs[numpy.newaxis].T
+
+        # We make sure that the correlation matrix is indeed diagonal:
+        assert (corr_mat-corr_mat.T).max() <= 1e-15
+        # We make sure that there are indeed ones on the diagonal:
+        assert (corr_mat.diagonal()-1).max() <= 1e-15
+
+        # We try to recover the correlated variables through the
+        # correlation matrix (not through the covariance matrix):
+
+        nominal_values = [v.nominal_value for v in (x, y, z)]
+        std_devs = [v.std_dev() for v in (x, y, z)]
+        x2, y2, z2 = uncertainties.correlated_values_norm(
+            zip(nominal_values, std_devs), corr_mat)
+        
+        # matrices_close() is used instead of _numbers_close() because
+        # it compares uncertainties too:
+
+        # Test of individual variables:
+        assert matrices_close(numpy.array([x]), numpy.array([x2]))
+        assert matrices_close(numpy.array([y]), numpy.array([y2]))
+        assert matrices_close(numpy.array([z]), numpy.array([z2]))
+
+        # Partial correlation test:
+        assert matrices_close(numpy.array([0]), numpy.array([z2-(-3*x2+y2)]))
+
+        # Test of the full covariance matrix:
+        assert matrices_close(
+            numpy.array(cov_mat),
+            numpy.array(uncertainties.covariance_matrix([x2, y2, z2])))
+
