@@ -28,7 +28,7 @@ The numbers with uncertainties handled by this module are objects from
 the uncertainties module, from either the Variable or the
 AffineScalarFunc class.
 
-(c) 2009-2010 by Eric O. LEBIGOT (EOL) <eric.lebigot@normalesup.org>.
+(c) 2009-2013 by Eric O. LEBIGOT (EOL) <eric.lebigot@normalesup.org>.
 Please send feature requests, bug reports, or feedback to this address.
 
 This software is released under a dual license.  (1) The BSD license.
@@ -77,13 +77,17 @@ from uncertainties import __author__, to_affine_scalar, AffineScalarFunc
 
 # Math functions that have a standard interface: they take
 # one or more float arguments, and return a scalar:
-many_scalar_to_scalar_funcs = []
+many_scalars_to_scalar_funcs = []
 
 # Some functions require a specific treatment and must therefore be
 # excluded from standard wrapping.  Functions
-no_std_wrapping = ['modf', 'frexp', 'ldexp', 'fsum', 'factorial']
+# no_std_wrapping = ['modf', 'frexp', 'ldexp', 'fsum', 'factorial']
 
-# Functions that do not belong in many_scalar_to_scalar_funcs, but
+# Functions with numerical derivatives:
+num_deriv_funcs = ['fmod', 'gamma', 'isinf', 'isnan',
+                   'lgamma', 'trunc']
+
+# Functions that do not belong in many_scalars_to_scalar_funcs, but
 # that have a version that handles uncertainties:
 non_std_wrapped_funcs = []
 
@@ -93,7 +97,7 @@ wraps = functools.partial(functools.update_wrapper,
                           assigned=('__doc__', '__name__'))
 
 ########################################
-# Wrapping of built-in math functions not in no_std_wrapping:
+# Wrapping of math functions:
 
 # Fixed formulas for the derivatives of some functions from the math
 # module (some functions might not be present in all version of
@@ -106,9 +110,10 @@ wraps = functools.partial(functools.update_wrapper,
 
 # Functions that have singularities (possibly at infinity) benefit
 # from analytical calculations (instead of the default numerical
-# calculation).  Even slowly varying functions (e.g., abs()) yield
-# more precise results when differentiated analytically, because of
-# the loss of precision in numerical calculations.
+# calculation) because their derivatives generally change very fast.
+# Even slowly varying functions (e.g., abs()) yield more precise
+# results when differentiated analytically, because of the loss of
+# precision in numerical calculations.
 
 #def log_1arg_der(x):
 #    """
@@ -135,7 +140,9 @@ def log_der0(*args):
     #    return log_1arg_der(*args)  # Argument number check
     #except TypeError:
     #    return 1/args[0]/math.log(args[1])  # 2-argument form
-    
+
+_erf_coef = 2/math.sqrt(math.pi)  # Optimization for erf()
+
 fixed_derivatives = {
     # In alphabetical order, here:
     'acos': [lambda x: -1/math.sqrt(1-x**2)],
@@ -152,7 +159,10 @@ fixed_derivatives = {
     'cos': [lambda x: -math.sin(x)],
     'cosh': [math.sinh],
     'degrees': [lambda x: math.degrees(1)],
+    'erf': [lambda x: exp(-x**2)*_erf_coef],
+    'erfc': [lambda x: -exp(-x**2)*_erf_coef],
     'exp': [math.exp],
+    'expm1': [math.exp],
     'fabs': [lambda x: 1 if x >= 0 else -1],
     'floor': [lambda x: 0],
     'hypot': [lambda x, y: x/math.hypot(x, y),
@@ -176,21 +186,24 @@ fixed_derivatives = {
 
 this_module = sys.modules[__name__]
 
-# We do not want to wrap module attributes such as __doc__, etc.:
-for (name, func) in inspect.getmembers(math, inspect.isbuiltin):
+# for (name, attr) in vars(math).items():
+for name in dir(math):
 
-    if name in no_std_wrapping:
-        continue
-
-    if name in fixed_derivatives:
+    if name in fixed_derivatives:  # Priority to functions in fixed_derivatives
         derivatives = fixed_derivatives[name]
-    else:
+    elif name in num_deriv_funcs:
         # Functions whose derivatives are calculated numerically by
         # this module fall here (isinf, fmod,...):
         derivatives = None  # Means: numerical calculation required
+    else:
+        continue  # 'name' not wrapped by this module (__doc__, e, etc.)
+
+    func = getattr(math, name)
+    
     setattr(this_module, name,
             wraps(uncertainties.wrap(func, derivatives), func))
-    many_scalar_to_scalar_funcs.append(name)
+    
+    many_scalars_to_scalar_funcs.append(name)
 
 ###############################################################################
     
@@ -267,7 +280,7 @@ def modf(x):
         # argument: there is no need to return numbers with uncertainties:
         return (frac_part, int_part)
     
-many_scalar_to_scalar_funcs.append('modf')
+many_scalars_to_scalar_funcs.append('modf')
 
 @uncertainties.set_doc(math.ldexp.__doc__)
 def ldexp(x, y):
@@ -298,7 +311,7 @@ def ldexp(x, y):
         # value of x coerced to a difference type [int->float, for
         # instance]):
         return math.ldexp(x, y)
-many_scalar_to_scalar_funcs.append('ldexp')
+many_scalars_to_scalar_funcs.append('ldexp')
 
 @uncertainties.set_doc(math.frexp.__doc__)
 def frexp(x):
@@ -335,5 +348,5 @@ non_std_wrapped_funcs.append('frexp')
 ###############################################################################
 # Exported functions:
 
-__all__ = many_scalar_to_scalar_funcs + non_std_wrapped_funcs
+__all__ = many_scalars_to_scalar_funcs + non_std_wrapped_funcs
 
